#pragma once        // MSVC
#ifndef PGCHELPER_H // QT
//! \cond
#define PGCHELPER_H
//! \endcond

//! \file
//! Payment Gateway Client Helper
//! \n Strings are all in UTF-8 encoding

////////////////////////////////////////////////////////////////////////////////

#include "PgcClient.h"
#include "PgcKeys.h"

#include <string>
#include <algorithm>

////////////////////////////////////////////////////////////////////////////////

//! \name MacroGetSec
//! Getter Macros section
//!@{

//! Double-get string value
//! \param[in] fnGetStr Function to get the value
//! \param[in] fnAssignStr Function to assign the value
//! \param[in] pVal Buffer parameter in the function
//! \param[in] nSize Size parameter in the function
//! \param[in] sVal Variable to receive the value
//! \return String value
#define PGC_DBLGET_STR(fnGetStr, fnAssignStr, pVal, nSize, sVal)        \
    {                                                                   \
        char * (pVal) = NULL; /* pVal is a var used in func too */      \
        int (nSize) = 0; /* nSize is a var used in func too */          \
                                                                        \
        (nSize) = (fnGetStr);                                           \
        if ((nSize) > 0)                                                \
        {                                                               \
            (pVal) = new char[nSize];                                   \
            (nSize) = (fnGetStr);                                       \
            (sVal) = (fnAssignStr);                                     \
            delete[] (pVal);                                            \
        }                                                               \
        else                                                            \
        {                                                               \
            (pVal) = ""; /* empty string on error */                    \
            (sVal) = (fnAssignStr);                                     \
        }                                                               \
    }


//! Short-get string value (size < 256)
//! \param[in] fnGetStr Function to get the value
//! \param[in] fnAssignStr Function to assign the value
//! \param[in] pVal Buffer parameter in the function
//! \param[in] nSize Size parameter in the function
//! \param[in] sVal Variable to receive the value
//! \return String value
#define PGC_SHTGET_STR(fnGetStr, fnAssignStr, pVal, nSize, sVal)    \
    {                                                               \
        int (nSize) = 256; /* nSize is a var used in func too */    \
        char (pVal)[256] = "";                                      \
        (nSize) = (fnGetStr);                                       \
        (sVal) = (fnAssignStr);                                     \
    }

//!@}


//! \name MacroWinCnvSec
//! Windows string converter Macros section
//!@{

//! Convert from VC TCHAR string to UTF-8 Unicode C string
//! \param[in] tstr VC TCHAR string
//! \return UTF-8 Unicode C string
#define PGC_CT2A(tstr) ((LPSTR)CT2A((tstr), CP_UTF8))


//! Convert from UTF-8 Unicode C string to VC TCHAR string
//! \param[in] str UTF-8 Unicode C string
//! \return VC TCHAR string
#define PGC_CA2T(str) ((LPTSTR)CA2T(str))


//! Convert from UTF-8 Unicode C++ std::string to VC TCHAR string
//! \param[in] str UTF-8 Unicode C++ std::string
//! \return VC TCHAR string
#define PGC_CS2T(str) (PGC_CA2T((str).c_str()))


//! Convert from Windows wide char string to UTF-8 Unicode string
//! \param[in] wstr Windows wide char string
//! \return UTF-8 Unicode string
#define PGC_CW2A(wstr) ((LPSTR)CW2A((wstr), CP_UTF8))


//! Convert from UTF-8 Unicode string to Windows wide char string
//! \param[in] str UTF-8 Unicode string
//! \return Windows wide char string
#define PGC_CA2W(str) ((LPWSTR)CA2W(str))


//! Convert from Windows system string (BSTR) to UTF-8 Unicode string
//! \param[in] bstr Windows system string
//! \return UTF-8 Unicode string
#define PGC_CB2A(bstr) (PGC_CW2A(CComBSTR(bstr)))


//! Convert from UTF-8 Unicode string to Windows system string
//! \param[in] str UTF-8 Unicode string
//! \return Windows system string
#define PGC_CA2B(str) (CComBSTR(str))

//!@}


//! \name MacroCnvSec
//! Converter Macros section
//!@{

//! Convert from a number value to string
//! \param[in] nVal Number value
//! \return String as std::string
#define PGC_TOSTR(nVal) (std::to_string(nVal))


//! Convert from a number value to string
//! \param[in] nVal Number value
//! \return String as const char *
#define PGC_TOCSTR(nVal) (PGC_TOSTR(nVal).c_str())


//! Convert number to amount (fixed 2 decimal places)
//! \param[in] nVal Number value
//! \return Amount value
#define PGC_NTOAMT(nVal) ((long)((nVal) * 100 + 0.5))


//! Convert string to amount (fixed 2 decimal places)
//! \param[in] sVal String value
//! \return Amount value
#define PGC_ATOAMT(sVal) PGC_NTOAMT(std::atof(sVal))


//! Convert string to integer
//! \param[in] sVal C string value
//! \return Integer value
#define PGC_ATOINT(sVal) (std::atoi(sVal))


//! Convert string to integer
//! \param[in] sVal String value
//! \return Integer value
#define PGC_STOINT(sVal) PGC_ATOINT((sVal).c_str())


//! Convert string to 64bit integer
//! \param[in] sVal C string value
//! \return Integer value
#define PGC_ATOLLINT(sVal) (std::atoll(sVal))


//! Convert string to 64bit integer
//! \param[in] sVal String value
//! \return Integer value
#define PGC_STOLLINT(sVal) PGC_ATOLLINT((sVal).c_str())


//! Convert string to boolean
//! \param[in] sVal C string value
//! \return Boolean value
#define PGC_ATOBOOL(sVal) (_stricmp((sVal), PGC_GENVAL_TRUE) == 0)


//! Convert string to boolean
//! \param[in] sVal String value
//! \return Boolean value
#define PGC_STOBOOL(sVal) PGC_ATOBOOL((sVal).c_str())


//! Convert boolean to string
//! \param[in] bVal Boolean value
//! \return String value
#define PGC_BOOLTOA(bVal) ((bVal) ? PGC_GENVAL_TRUE : PGC_GENVAL_FALSE)


//! Convert integer to boolean
//! \param[in] nVal Integer value
//! \return Boolean value
#define PGC_INTTOBOOL(nVal) ((nVal) != 0)


//! Convert boolean to integer
//! \param[in] bVal Boolean value
//! \return Integer value
#define PGC_BOOLTOINT(bVal) ((bVal) ? 1 : 0)


//! Convert boolean to integer string
//! \param[in] bVal Boolean value
//! \return String value
#define PGC_BOOLTOINTSTR(bVal) ((bVal) ? "1" : "0")


//! Ensure value is not null
//! \param[in] val Value to pass in
//! \param[in] def Default value
//! \return Any no null value
#define PGC_NONULL(val, def) ((val) ? (val) : (def))


//! Ensure value is not null
//! \param[in] sVal Value to pass in
//! \return Any no null value
#define PGC_NONULLSTR(sVal) PGC_NONULL((sVal), PGC_GENVAL_EMPTY)

//!@}

////////////////////////////////////////////////////////////////////////////////

//! \name HlpFuncsSec
//! Functions section
//!@{

//! Helper
class PgcHelper
{
public: // work functions
    //! Set the raw string to the session (entire message just contains this one raw string; nothing else)
    //! \param[in] nSess Session
    //! \param[in] sVal String value
    //! \return String value
    static int PgcSetRawStr(int nSess, const char * sVal)
    {
        //return PgcSetStr(nSess, 0, PGC_MSGKEY_RAW, sVal);
        return PgcSetMsgData(nSess, true, sVal);
    }


    //! Get the raw string from the session (entire message just contains this one raw string; nothing else)
    //! \param[in] nSess Session
    //! \return String value
    static std::string PgcGetRawStr2(int nSess)
    {
        //return PgcGetStr2(nSess, 0, PGC_MSGKEY_RAW);
        return PgcGetMsgData2(nSess, true);
    }


    //! Get error tag (description of error)
    //! \param[in] nErrCode Error code
    //! \return String value
    static std::string PgcGetErrTag2(int nErrCode)
    {
        char sTag[256] = "";
        PgcGetErrTag(nErrCode, sTag);
        std::string sRst = sTag;
        return sRst;
    }


    //! Get connection property
    //! \param[in] nConn Connection
    //! \param[in] sSec Section of the data
    //! \param[in] sKey Key of the data
    //! \param[in] sDefVal Default value of the data
    //! \return String value
    static std::string PgcGetConnProp2(int nConn, const char * sSec, const char * sKey, const char * sDefVal)
    {
        std::string sVal;
        PGC_DBLGET_STR(PgcGetConnProp(nConn, sSec, sKey, sDefVal, pVal, nSize), (pVal), pVal, nSize, sVal);
        return sVal;
    }


    //! Get connection status/statistics
    //! \param[in] nConn Connection
    //! \param[in] sKey Key of the data
    //! \return String value
    static std::string PgcGetConnSts2(int nConn, const char * sKey)
    {
        std::string sVal;
        PGC_DBLGET_STR(PgcGetConnSts(nConn, sKey, pVal, nSize), (pVal), pVal, nSize, sVal);
        return sVal;
    }


    //! Get session status/statistics
    //! \param[in] nSess Session
    //! \param[in] sKey Key of the data
    //! \return String value
    static std::string PgcGetSessSts2(int nSess, const char * sKey)
    {
        std::string sVal;
        PGC_DBLGET_STR(PgcGetSessSts(nSess, sKey, pVal, nSize), (pVal), pVal, nSize, sVal);
        return sVal;
    }


    //! Get message parameter
    //! \param[in] nSess Session
    //! \param[in] sKey Key of the data
    //! \return String value
    static std::string PgcGetMsgPrm2(int nSess, const char * sKey)
    {
        std::string sVal;
        PGC_DBLGET_STR(PgcGetMsgPrm(nSess, sKey, pVal, nSize), (pVal), pVal, nSize, sVal);
        return sVal;
    }


    //! Get string value
    //! \param[in] nSess Session
    //! \param[in] nMsg Message index
    //! \param[in] sKey Key of the data
    //! \return String value
    static std::string PgcGetStr2(int nSess, int nMsg, const char * sKey)
    {
        std::string sVal;
        PGC_DBLGET_STR(PgcGetStr(nSess, nMsg, sKey, pVal, nSize), (pVal), pVal, nSize, sVal);
        return sVal;
    }


    //! Get message data
    //! \param[in] nSess Session
    //! \param[in] bIsRaw If the output should be the raw string
    //! \return String value
    static std::string PgcGetMsgData2(int nSess, bool bIsRaw)
    {
        std::string sVal;
        PGC_DBLGET_STR(PgcGetMsgData(nSess, bIsRaw, pVal, nSize), (pVal), pVal, nSize, sVal);
        return sVal;
    }


    //! Get connection property; short version
    //! \param[in] nConn Connection
    //! \param[in] sSec Section of the data
    //! \param[in] sKey Key of the data
    //! \param[in] sDefVal Default value of the data
    //! \return String value
    static std::string PgcGetConnPropS(int nConn, const char * sSec, const char * sKey, const char * sDefVal)
    {
        std::string sVal;
        PGC_SHTGET_STR(PgcGetConnProp(nConn, sSec, sKey, sDefVal, pVal, nSize), (pVal), pVal, nSize, sVal);
        return sVal;
    }


    //! Get connection status/statistics; short version
    //! \param[in] nConn Connection
    //! \param[in] sKey Key of the data
    //! \return String value
    static std::string PgcGetConnStsS(int nConn, const char * sKey)
    {
        std::string sVal;
        PGC_SHTGET_STR(PgcGetConnSts(nConn, sKey, pVal, nSize), (pVal), pVal, nSize, sVal);
        return sVal;
    }


    //! Get connection status/statistics as integer
    //! \param[in] nConn Connection
    //! \param[in] sKey Key of the data
    //! \return int value
    static int PgcGetConnStsInt(int nConn, const char * sKey)
    {
        return PGC_STOINT(PgcGetConnStsS(nConn, sKey));
    }


    //! Get connection status/statistics as boolean
    //! \param[in] nConn Connection
    //! \param[in] sKey Key of the data
    //! \return bool value
    static int PgcGetConnStsBool(int nConn, const char * sKey)
    {
        return PGC_STOBOOL(PgcGetConnStsS(nConn, sKey));
    }


    //! Get session status/statistics; short version
    //! \param[in] nSess Session
    //! \param[in] sKey Key of the data
    //! \return String value
    static std::string PgcGetSessStsS(int nSess, const char * sKey)
    {
        std::string sVal;
        PGC_SHTGET_STR(PgcGetSessSts(nSess, sKey, pVal, nSize), (pVal), pVal, nSize, sVal);
        return sVal;
    }


    //! Get message parameter; short version
    //! \param[in] nSess Session
    //! \param[in] sKey Key of the data
    //! \return String value
    static std::string PgcGetMsgPrmS(int nSess, const char * sKey)
    {
        std::string sVal;
        PGC_SHTGET_STR(PgcGetMsgPrm(nSess, sKey, pVal, nSize), (pVal), pVal, nSize, sVal);
        return sVal;
    }


    //! Get string value; short version
    //! \param[in] nSess Session
    //! \param[in] nMsg Message index
    //! \param[in] sKey Key of the data
    //! \return String value
    static std::string PgcGetStrS(int nSess, int nMsg, const char * sKey)
    {
        std::string sVal;
        PGC_SHTGET_STR(PgcGetStr(nSess, nMsg, sKey, pVal, nSize), (pVal), pVal, nSize, sVal);
        return sVal;
    }


    //! Replace content in a string
    //! \param str String to operate
    //! \param[in] from String to be replaced from
    //! \param[in] to String to be replaced to
    //! \return True if successful
    static bool StrReplace(std::string & str, const std::string & from, const std::string & to)
    {
        size_t start_pos = str.find(from);
        if (start_pos == std::string::npos)
            return false;
        str.replace(start_pos, from.length(), to);
        return true;
    }


    //! Replace content in a string
    //! \param str String to operate
    //! \param[in] from String to be replaced from
    //! \param[in] to String to be replaced to
    //! \return True if successful
    static bool StrReplace(std::string & str, const char * from, const std::string & to)
    {
        std::string sFrom = from;
        return StrReplace(str, sFrom, to);
    }


    //! Convert string to lower case
    //! \param str String to operate
    static void StrToLower(std::string & str)
    {
        std::transform(str.begin(), str.end(), str.begin(), ::tolower);
    }
};

//!@}

#endif // PGCHELPER_H
